import { world, system } from "@minecraft/server";
import { ModalFormData } from "@minecraft/server-ui";

const CONFIG_KEY = "scoreboardConfig";
const ADMIN_TAG = "admin";

// Inicialize o objeto SCOREBOARD com 15 linhas vazias
let SCOREBOARD = {
    linha1: "",
    linha2: "",
    linha3: "",
    linha4: "",
    linha5: "",
    linha6: "",
    linha7: "",
    linha8: "",
    linha9: "",
    linha10: "",
    linha11: "",
    linha12: "",
    linha13: "",
    linha14: "",
    linha15: ""
};

// Função para carregar configuração
function loadConfig() {
    const data = world.getDynamicProperty(CONFIG_KEY);
    if (data) {
        SCOREBOARD = JSON.parse(data);
    }
}

// Função de salvamento de configurações
function saveConfig() {
    world.setDynamicProperty(CONFIG_KEY, JSON.stringify(SCOREBOARD));
}

// Função para mostrar o menu de administração e editar a SCOREBOARD
function showAdminMenu(player) {
    const form = new ModalFormData()
        .title(`§a%scoreboard.title.menu`);

    Object.keys(SCOREBOARD).forEach((line, index) => {
        form.textField(`§g%scoreboard.line.menu §7${index + 1}`, SCOREBOARD[line] || "");
    });

    form.show(player).then((response) => {
        if (response.canceled) return;

        Object.keys(SCOREBOARD).forEach((line, index) => {
            SCOREBOARD[line] = response.formValues[index] || "";
        });

        saveConfig();
        player.sendMessage("§a%scoreboard.current.menu");
    });
}

// Função para substituir %score% pelo valor da SCOREBOARD
function replaceScorePlaceholders(text, player) {
    return text.replace(/%(\w+)%/g, (match, p1) => {
        const score = getScore(player, p1);
        return score !== undefined ? score : match;
    });
}

// Função para substituir {PLAYER} pelo nome do jogador
function replacePlayerName(text, player) {
    return text.replace(/{PLAYER}/g, player.name);
}

// Obtenha o valor de uma pontuação
function getScore(player, objective) {
    try {
        return world.scoreboard.getObjective(objective)?.getScore(player) || 0;
    } catch {
        player.runCommand(`scoreboard objectives add ${objective} dummy`);
        player.runCommand(`scoreboard players set @s ${objective} 0`);
        return 0;
    }
}

// Mostra a SCOREBOARD para um jogador usando titleraw
function showScoreboard(player) {
    let scoreboardText = "";
    Object.keys(SCOREBOARD).forEach(line => {
        const lineText = SCOREBOARD[line];
        if (lineText) {
            scoreboardText += replaceScorePlaceholders(replacePlayerName(lineText, player), player) + "\\n";
        }
    });

    scoreboardText = scoreboardText.trim();
    scoreboardText = scoreboardText.replace(/\\n$/, ""); // Remover caractere de nova linha final

    try {
        player.runCommand(`titleraw @s title {"rawtext":[{"text":"${scoreboardText}"}]}`);
    } catch (error) {
        player.sendMessage("§cErro ao exibir a ScoreBoard.");
    }
}

// Evento para abrir o menu de administração
world.afterEvents.itemUse.subscribe(event => {
    const player = event.source;
    const item = event.itemStack;
    if (player.hasTag(ADMIN_TAG) && item.typeId === "minexraft:compass") {
        showAdminMenu(player);
    }
});

// Carregar configuração na inicialização
world.afterEvents.worldInitialize.subscribe(() => {
    loadConfig();
});

system.runInterval(() => {
    world.getAllPlayers().forEach(player => {
        let scoreboardText = "";
        Object.keys(SCOREBOARD).forEach(line => {
            const lineText = SCOREBOARD[line];
            if (lineText) {
                scoreboardText += replaceScorePlaceholders(replacePlayerName(lineText, player), player) + "\\n";
            }
        });
    
        scoreboardText = scoreboardText.trim();
        scoreboardText = scoreboardText.replace(/\\n$/, ""); // Remover caractere de nova linha final
    
        try {
            player.runCommand(`titleraw @s title {"rawtext":[{"text":"${scoreboardText}"}]}`);
        } catch (error) {
            player.sendMessage("§cErro ao exibir a ScoreBoard.");
        }
    });
}, 20); // Mostrar a cada 10 segundos